import slugger from 'github-slugger';
/**
 * Create Markdown Headers Collector
 * NOTE: micromark has terrible TS types. Instead of fighting with the
 * limited/broken TS types that they ship, we just reach for our good friend, "any".
 */
export function createMarkdownHeadersCollector() {
    const headers = [];
    let currentHeader;
    return {
        headers,
        headersExtension: {
            enter: {
                atxHeading(node) {
                    currentHeader = {};
                    headers.push(currentHeader);
                    this.buffer();
                },
                atxHeadingSequence(node) {
                    currentHeader.depth = this.sliceSerialize(node).length;
                },
                atxHeadingText(node) {
                    currentHeader.text = this.sliceSerialize(node);
                },
            },
            exit: {
                atxHeading(node) {
                    currentHeader.slug = slugger.slug(currentHeader.text);
                    this.resume();
                    this.tag(`<h${currentHeader.depth} id="${currentHeader.slug}">`);
                    this.raw(currentHeader.text);
                    this.tag(`</h${currentHeader.depth}>`);
                },
            },
        },
    };
}
