/// <reference types="node" />
import type { SnowpackDevServer, ServerRuntime as SnowpackServerRuntime, SnowpackConfig } from 'snowpack';
import type { AstroConfig, RuntimeMode } from './@types/astro';
import type { LogOptions } from './logger';
import type { CompileError } from './parser/utils/error.js';
interface RuntimeConfig {
    astroConfig: AstroConfig;
    logging: LogOptions;
    mode: RuntimeMode;
    backendSnowpack: SnowpackDevServer;
    backendSnowpackRuntime: SnowpackServerRuntime;
    backendSnowpackConfig: SnowpackConfig;
    frontendSnowpack: SnowpackDevServer;
    frontendSnowpackRuntime: SnowpackServerRuntime;
    frontendSnowpackConfig: SnowpackConfig;
}
declare type LoadResultSuccess = {
    statusCode: 200;
    contents: string | Buffer;
    contentType?: string | false;
};
declare type LoadResultNotFound = {
    statusCode: 404;
    error: Error;
};
declare type LoadResultRedirect = {
    statusCode: 301 | 302;
    location: string;
};
declare type LoadResultError = {
    statusCode: 500;
} & ({
    type: 'parse-error';
    error: CompileError;
} | {
    type: 'unknown';
    error: Error;
});
export declare type LoadResult = LoadResultSuccess | LoadResultNotFound | LoadResultRedirect | LoadResultError;
export interface AstroRuntime {
    runtimeConfig: RuntimeConfig;
    load: (rawPathname: string | undefined) => Promise<LoadResult>;
    shutdown: () => Promise<void>;
}
interface RuntimeOptions {
    mode: RuntimeMode;
    logging: LogOptions;
}
/** Core Astro runtime */
export declare function createRuntime(astroConfig: AstroConfig, { mode, logging }: RuntimeOptions): Promise<AstroRuntime>;
export {};
